﻿/*
 *  TOPPERS/ECNL Prototyping tool
 * 
 *  Copyright (C) 2015-2017 Cores Co., Ltd. Japan
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: CodeGenerator.cs 287 2017-05-05 14:22:23Z coas-nagasima $
 */

using System;
using System.Linq;
using System.Text;
using Bridge;
using Bridge.Text.RegularExpressions;

namespace WebMrbc
{
	public static class CodeGenerator
	{
		public static string GetClassIdentifier(JsonClassInfo cls)
		{
			string result = "";
			bool up = true;
			foreach (var c in cls.name.ToLower()) {
				if (c == '_') {
					up = true;
					continue;
				}
				result += up ? c.ToString().ToUpper() : c.ToString();
				up = false;
			}
			if (result.EndsWith("Eobj"))
				result = result.Substring(0, result.Length - 4);

			if (!new Regex("^[A-Z][A-Za-z0-9_]+$").Test(result)) {
				result = String.Format("Cls{0:X2}{1:X2}", cls.classGroup.classGroupCode, cls.classCode);
			}

			return result;
		}

		public static string GetPropertyIdentifier(JsonPropertyInfo pi)
		{
			JsonObjectInfo cls = pi.classInfo;
			JsonFieldInfo fi = pi;
			string result = fi.identifier;

			if (!new Regex("^[A-Za-z_][A-Za-z0-9_]+$").Test(result)) {
				result = String.Format("property{0:X2}", pi.propertyCode);
			}
			else {
				var reg = new Regex("[A-Z][a-z]+", "g");
				string[] m;
				if ((m = reg.Exec(result)) != null) {
					var blocks = new string[0];
					int pos = 0;
					do {
						int len = Script.Write<int>("m.index") + m[0].Length - pos;
						blocks.Push(result.Substring(pos, len).ToLower());
						pos += len;
					} while ((m = reg.Exec(result)) != null);
					result = blocks.Join("_");
				}
			}

			return result;
		}

		internal static bool HasPropSetter(JsonPropertyInfo epi, out ValueRange valueRange)
		{
			valueRange = null;

			if (!epi.primitive || (epi.arrayCount != 0))
				return false;

			valueRange = ValueRange.Parse(epi.valueDescription, epi);
			if ((valueRange.Values.Length > 0) || (valueRange.Ranges.Length > 0)) {
				return true;
			}

			valueRange = null;
			return false;
		}

		internal static string GetInitialValue(string valRng, JsonFieldInfo pi)
		{
			ValueRange valueRange = ValueRange.Parse(valRng, pi);
			string init = "0000000000000000" + valueRange.InitailValue;

			char[] temp = new char[4 * pi.size];
			for (int i = 0, j = init.Length - 2 * pi.size; i < temp.Length; i++, j++) {
				if ((i % 4) == 0) {
					temp[i++] = '\\';
					temp[i++] = 'x';
				}
				temp[i] = init[j];
			}
			return new String(temp);
		}

		internal static bool IsExtractProperty(JsonPropertyInfo prp)
		{
			switch (prp.propertyCode) {
			case 0x9B:  // ＳｅｔＭプロパティマップ
			case 0x9C:  // ＧｅｔＭプロパティマップ
						// はECHONET Liteにはないので作らない
				return true;
			case 0x9D:  // 状変アナウンスプロパティマップ
			case 0x9E:  // Ｓｅｔプロパティマップ
			case 0x9F:  // Ｇｅｔプロパティマップ
			case 0xD3:  // 自ノードインスタンス数
			case 0xD4:  // 自ノードクラス数
			case 0xD5:  // インスタンスリスト通知
			case 0xD6:  // 自ノードインスタンスリストＳ
			case 0xD7:  // 自ノードクラスリストＳ
						// はコンフィグレーターで生成するので作らない
				return true;
			}

			return false;
		}
	}
}